@@ -20,6 +20,9 @@
 #define jit_arg_reg_p(i)		((i) >= 0 && (i) < 8)
 #define jit_arg_f_reg_p(i)		((i) >= 0 && (i) < 8)
 
+#if __APPLE__
+typedef jit_pointer_t jit_va_list_t;
+#else
 typedef struct jit_qreg {
     jit_float64_t	l;
     jit_float64_t	h;
@@ -52,6 +55,7 @@ typedef struct jit_va_list {
     jit_qreg_t		q6;
     jit_qreg_t		q7;
 } jit_va_list_t;
+#endif
 
 /*
  * Prototypes
@@ -72,7 +76,11 @@ extern void __clear_cache(void *, void *);
  */
 jit_register_t		_rvs[] = {
     { rc(gpr) | 0x08,			"x8" },
+#if __APPLE__
+    { 0x12,				"x18" },
+#else
     { rc(gpr) | 0x12,			"x18" },
+#endif
     { rc(gpr) | 0x11,			"x17" },
     { rc(gpr) | 0x10,			"x16" },
     { rc(gpr) | 0x09,			"x9" },
@@ -258,20 +266,18 @@ _jit_ret(jit_state_t *_jit)
 }
 
 void
-_jit_retr(jit_state_t *_jit, jit_int32_t u)
+_jit_retr(jit_state_t *_jit, jit_int32_t u, jit_code_t code)
 {
-    jit_inc_synth_w(retr, u);
-    if (JIT_RET != u)
-	jit_movr(JIT_RET, u);
-    jit_live(JIT_RET);
+    jit_code_inc_synth_w(code, u);
+    jit_movr(JIT_RET, u);
     jit_ret();
     jit_dec_synth();
 }
 
 void
-_jit_reti(jit_state_t *_jit, jit_word_t u)
+_jit_reti(jit_state_t *_jit, jit_word_t u, jit_code_t code)
 {
-    jit_inc_synth_w(reti, u);
+    jit_code_inc_synth_w(code, u);
     jit_movi(JIT_RET, u);
     jit_ret();
     jit_dec_synth();
@@ -331,7 +337,7 @@ _jit_epilog(jit_state_t *_jit)
 jit_bool_t
 _jit_arg_register_p(jit_state_t *_jit, jit_node_t *u)
 {
-    if (u->code == jit_code_arg)
+    if (u->code >= jit_code_arg_c && u->code <= jit_code_arg)
 	return (jit_arg_reg_p(u->u.w));
     assert(u->code == jit_code_arg_f || u->code == jit_code_arg_d);
     return (jit_arg_f_reg_p(u->u.w));
@@ -351,6 +357,7 @@ _jit_ellipsis(jit_state_t *_jit)
 	assert(!(_jitc->function->self.call & jit_call_varargs));
 	_jitc->function->self.call |= jit_call_varargs;
 
+#if !__APPLE_
 	/* Allocate va_list like object in the stack,
 	 * with enough space to save all argument
 	 * registers, and use fixed offsets for them. */
@@ -367,6 +374,7 @@ _jit_ellipsis(jit_state_t *_jit)
 	    _jitc->function->vafp = (8 - _jitc->function->self.argf) * -16;
 	else
 	    _jitc->function->vafp = 0;
+#endif
     }
     jit_dec_synth();
 }
@@ -380,7 +388,7 @@ _jit_va_push(jit_state_t *_jit, jit_int32_t u)
 }
 
 jit_node_t *
-_jit_arg(jit_state_t *_jit)
+_jit_arg(jit_state_t *_jit, jit_code_t code)
 {
     jit_node_t		*node;
     jit_int32_t		 offset;
@@ -389,10 +397,21 @@ _jit_arg(jit_state_t *_jit)
     if (jit_arg_reg_p(_jitc->function->self.argi))
 	offset = _jitc->function->self.argi++;
     else {
+#if PACKED_STACK || STRONG_TYPE_CHECKING
+	assert(code >= jit_code_arg_c && code <= jit_code_arg);
+#endif
+#if PACKED_STACK
+	_jitc->function->self.size +=
+	    _jitc->function->self.size & ((1 << (code - jit_code_arg_c)) - 1);
+#endif
 	offset = _jitc->function->self.size;
+#if PACKED_STACK
+	_jitc->function->self.size += 1 << (code - jit_code_arg_c);
+#else
 	_jitc->function->self.size += sizeof(jit_word_t);
+#endif
     }
-    node = jit_new_node_ww(jit_code_arg, offset,
+    node = jit_new_node_ww(code, offset,
 			   ++_jitc->function->self.argn);
     jit_link_prolog();
     return (node);
@@ -408,8 +427,16 @@ _jit_arg_f(jit_state_t *_jit)
     if (jit_arg_f_reg_p(_jitc->function->self.argf))
 	offset = _jitc->function->self.argf++;
     else {
+#if PACKED_STACK
+	_jitc->function->self.size +=
+	    _jitc->function->self.size & (sizeof(jit_float32_t) - 1);
+#endif
 	offset = _jitc->function->self.size;
+#if PACKED_STACK
+	_jitc->function->self.size += sizeof(jit_float32_t);
+#else
 	_jitc->function->self.size += sizeof(jit_word_t);
+#endif
     }
     node = jit_new_node_ww(jit_code_arg_f, offset,
 			   ++_jitc->function->self.argn);
@@ -427,8 +454,12 @@ _jit_arg_d(jit_state_t *_jit)
     if (jit_arg_f_reg_p(_jitc->function->self.argf))
 	offset = _jitc->function->self.argf++;
     else {
+#if PACKED_STACK
+	_jitc->function->self.size +=
+	    _jitc->function->self.size & (sizeof(jit_float64_t) - 1);
+#endif
 	offset = _jitc->function->self.size;
-	_jitc->function->self.size += sizeof(jit_word_t);
+	_jitc->function->self.size += sizeof(jit_float64_t);
     }
     node = jit_new_node_ww(jit_code_arg_d, offset,
 			   ++_jitc->function->self.argn);
@@ -439,10 +470,15 @@ _jit_arg_d(jit_state_t *_jit)
 void
 _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_c);
     jit_inc_synth_wp(getarg_c, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_c(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_c(u, JIT_FP, v->u.w);
     jit_dec_synth();
@@ -451,10 +487,15 @@ _jit_getarg_c(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 void
 _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_c);
     jit_inc_synth_wp(getarg_uc, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_uc(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_uc(u, JIT_FP, v->u.w);
     jit_dec_synth();
@@ -463,10 +504,15 @@ _jit_getarg_uc(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 void
 _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_s);
     jit_inc_synth_wp(getarg_s, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_s(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_s(u, JIT_FP, v->u.w);
     jit_dec_synth();
@@ -475,10 +521,15 @@ _jit_getarg_s(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 void
 _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_s);
     jit_inc_synth_wp(getarg_us, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_us(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_us(u, JIT_FP, v->u.w);
     jit_dec_synth();
@@ -487,22 +538,33 @@ _jit_getarg_us(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 void
 _jit_getarg_i(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_i);
     jit_inc_synth_wp(getarg_i, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK || __WORDSIZE == 32
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_i(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_i(u, JIT_FP, v->u.w);
     jit_dec_synth();
 }
 
+#if __WORDSIZE == 64
 void
 _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_i);
     jit_inc_synth_wp(getarg_ui, u, v);
-    if (jit_arg_reg_p(v->u.w))
+    if (jit_arg_reg_p(v->u.w)) {
+#if PACKED_STACK
+	jit_movr(u, JIT_RA0 - v->u.w);
+#else
 	jit_extr_ui(u, JIT_RA0 - v->u.w);
+#endif
+    }
     else
 	jit_ldxi_ui(u, JIT_FP, v->u.w);
     jit_dec_synth();
@@ -511,7 +573,7 @@ _jit_getarg_ui(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 void
 _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 {
-    assert(v->code == jit_code_arg);
+    assert_arg_type(v->code, jit_code_arg_l);
     jit_inc_synth_wp(getarg_l, u, v);
     if (jit_arg_reg_p(v->u.w))
 	jit_movr(u, JIT_RA0 - v->u.w);
@@ -519,31 +581,106 @@ _jit_getarg_l(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
 	jit_ldxi_l(u, JIT_FP, v->u.w);
     jit_dec_synth();
 }
+#endif
 
 void
-_jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v)
+_jit_putargr(jit_state_t *_jit, jit_int32_t u, jit_node_t *v, jit_code_t code)
 {
-    assert(v->code == jit_code_arg);
-    jit_inc_synth_wp(putargr, u, v);
-    if (jit_arg_reg_p(v->u.w))
-	jit_movr(JIT_RA0 - v->u.w, u);
-    else
+    assert_putarg_type(code, v->code);
+    jit_code_inc_synth_wp(code, u, v);
+    if (jit_arg_reg_p(v->u.w)) {
+	jit_int32_t	regno = JIT_RA0 - v->u.w;
+#if PACKED_STACK
+	switch (code) {
+	    case jit_code_putargr_c:	jit_extr_c(regno, u);	break;
+	    case jit_code_putargr_uc:	jit_extr_uc(regno, u);	break;
+	    case jit_code_putargr_s:	jit_extr_s(regno, u);	break;
+	    case jit_code_putargr_us:	jit_extr_us(regno, u);	break;
+#  if __WORDISZE == 32
+	    case jit_code_putargr_i:	jit_movr(regno, u);	break;
+#  else
+	    case jit_code_putargr_i:	jit_extr_i(regno, u);	break;
+	    case jit_code_putargr_ui:	jit_extr_ui(regno, u);	break;
+	    case jit_code_putargr_l:	jit_movr(regno, u);	break;
+#  endif
+	    default:			abort();		break;
+	}
+#else
+	jit_movr(regno, u);
+#endif
+    }
+    else {
+#if PACKED_STACK
+	switch (code) {
+	    case jit_code_putargr_c:	case jit_code_putargr_uc:
+		jit_stxi_c(v->u.w, JIT_FP, u);			break;
+	    case jit_code_putargr_s:	case jit_code_putargr_us:
+		jit_stxi_s(v->u.w, JIT_FP, u);			break;
+#  if __WORDSIZE == 32
+	    case jit_code_putargr_i:
+		jit_stxi(v->u.w, JIT_FP, u);			break;
+#  else
+	    case jit_code_putargr_i:	case jit_code_putargr_ui:
+		jit_stxi_i(v->u.w, JIT_FP, u);			break;
+	    case jit_code_putargr_l:
+		jit_stxi(v->u.w, JIT_FP, u);			break;
+#  endif
+	    default:			abort();		break;
+	}
+#else
 	jit_stxi(v->u.w, JIT_FP, u);
+#endif
+    }
     jit_dec_synth();
 }
 
 void
-_jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v)
+_jit_putargi(jit_state_t *_jit, jit_word_t u, jit_node_t *v, jit_code_t code)
 {
     jit_int32_t		regno;
-    assert(v->code == jit_code_arg);
-    jit_inc_synth_wp(putargi, u, v);
+    assert_putarg_type(code, v->code);
+    jit_code_inc_synth_wp(code, u, v);
+#if PACKED_STACK
+    switch (code) {
+	case jit_code_putargi_c:	u = (jit_int8_t)u;	break;
+	case jit_code_putargi_uc:	u = (jit_uint8_t)u;	break;
+	case jit_code_putargi_s:	u = (jit_int16_t)u;	break;
+	case jit_code_putargi_us:	u = (jit_uint16_t)u;	break;
+#  if __WORDSIZE == 32
+	case jit_code_putargi_i:				break;
+#  else
+	case jit_code_putargi_i:	u = (jit_int32_t)u;	break;
+	case jit_code_putargi_ui:	u = (jit_uint32_t)u;	break;
+	case jit_code_putargi_l:				break;
+#  endif
+	default:			abort();		break;
+    }
+#endif
     if (jit_arg_reg_p(v->u.w))
 	jit_movi(JIT_RA0 - v->u.w, u);
     else {
 	regno = jit_get_reg(jit_class_gpr);
 	jit_movi(regno, u);
+#if PACKED_STACK
+	switch (code) {
+	    case jit_code_putargi_c:	case jit_code_putargi_uc:
+		jit_stxi_c(v->u.w, JIT_FP, regno);		break;
+	    case jit_code_putargi_s:	case jit_code_putargi_us:
+		jit_stxi_s(v->u.w, JIT_FP, regno);		break;
+#  if __WORDSIZE == 32
+	    case jit_code_putargi_i:
+		jit_stxi(v->u.w, JIT_FP, regno);		break;
+#  else
+	    case jit_code_putargi_i:	case jit_code_putargi_ui:
+		jit_stxi_i(v->u.w, JIT_FP, regno);		break;
+	    case jit_code_putargi_l:
+		jit_stxi(v->u.w, JIT_FP, regno);		break;
+#  endif
+	    default:			abort();		break;
+	}
+#else
 	jit_stxi(v->u.w, JIT_FP, regno);
+#endif
 	jit_unget_reg(regno);
     }
     jit_dec_synth();
@@ -632,39 +769,148 @@ _jit_putargi_d(jit_state_t *_jit, jit_float64_t u, jit_node_t *v)
 }
 
 void
-_jit_pushargr(jit_state_t *_jit, jit_int32_t u)
+_jit_pushargr(jit_state_t *_jit, jit_int32_t u, jit_code_t code)
 {
     assert(_jitc->function);
-    jit_inc_synth_w(pushargr, u);
+    jit_code_inc_synth_w(code, u);
     jit_link_prepare();
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	jit_movr(JIT_RA0 - _jitc->function->call.argi, u);
+	jit_int32_t	regno = JIT_RA0 - _jitc->function->call.argi;
+#if PACKED_STACK
+	switch (code) {
+	    case jit_code_pushargr_c:	jit_extr_c(regno, u);	break;
+	    case jit_code_pushargr_uc:	jit_extr_uc(regno, u);	break;
+	    case jit_code_pushargr_s:	jit_extr_s(regno, u);	break;
+	    case jit_code_pushargr_us:	jit_extr_us(regno, u);	break;
+#  if __WORDISZE == 32
+	    case jit_code_pushargr_i:	jit_movr(regno, u);	break;
+#  else
+	    case jit_code_pushargr_i:	jit_extr_i(regno, u);	break;
+	    case jit_code_pushargr_ui:	jit_extr_ui(regno, u);	break;
+	    case jit_code_pushargr_l:	jit_movr(regno, u);	break;
+#  endif
+	    default:			abort();		break;
+	}
+#else
+	jit_movr(regno, u);
+#endif
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(code == jit_code_pushargr);
+	    jit_stxi(_jitc->function->call.size, JIT_SP, u);
+	    _jitc->function->call.size += sizeof(jit_word_t);
+	}
+#endif
 	++_jitc->function->call.argi;
     }
     else {
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size &
+		((1 << ((code - jit_code_pushargr_c) >> 2)) - 1);
+	switch (code) {
+	    case jit_code_pushargr_c:	case jit_code_pushargr_uc:
+		jit_stxi_c(_jitc->function->call.size, JIT_SP, u);
+		break;
+	    case jit_code_pushargr_s:	case jit_code_pushargr_us:
+		jit_stxi_s(_jitc->function->call.size, JIT_SP, u);
+		break;
+#  if __WORDSIZE == 32
+	    case jit_code_pushargr_i:
+		jit_stxi(_jitc->function->call.size, JIT_SP, u);
+		break;
+#  else
+	    case jit_code_pushargr_i:	case jit_code_pushargr_ui:
+		jit_stxi_i(_jitc->function->call.size, JIT_SP, u);
+		break;
+	    case jit_code_pushargr_l:
+		jit_stxi(_jitc->function->call.size, JIT_SP, u);
+		break;
+#  endif
+	    default:
+		abort();
+		break;
+	}
+	_jitc->function->call.size += 1 << ((code - jit_code_pushargr_c) >> 2);
+#else
 	jit_stxi(_jitc->function->call.size, JIT_SP, u);
 	_jitc->function->call.size += sizeof(jit_word_t);
+#endif
     }
     jit_dec_synth();
 }
 
 void
-_jit_pushargi(jit_state_t *_jit, jit_word_t u)
+_jit_pushargi(jit_state_t *_jit, jit_word_t u, jit_code_t code)
 {
     jit_int32_t		 regno;
     assert(_jitc->function);
-    jit_inc_synth_w(pushargi, u);
+    jit_code_inc_synth_w(code, u);
     jit_link_prepare();
+#if PACKED_STACK
+    switch (code) {
+	case jit_code_pushargi_c:	u = (jit_int8_t)u;	break;
+	case jit_code_pushargi_uc:	u = (jit_uint8_t)u;	break;
+	case jit_code_pushargi_s:	u = (jit_int16_t)u;	break;
+	case jit_code_pushargi_us:	u = (jit_uint16_t)u;	break;
+#  if __WORDSIZE == 32
+	case jit_code_pushargi_i:				break;
+#  else
+	case jit_code_pushargi_i:	u = (jit_int32_t)u;	break;
+	case jit_code_pushargi_ui:	u = (jit_uint32_t)u;	break;
+	case jit_code_pushargi_l:				break;
+#  endif
+	default:			abort();		break;
+    }
+#endif
     if (jit_arg_reg_p(_jitc->function->call.argi)) {
-	jit_movi(JIT_RA0 - _jitc->function->call.argi, u);
+	regno = JIT_RA0 - _jitc->function->call.argi;
+	jit_movi(regno, u);
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(code == jit_code_pushargi);
+	    jit_stxi(_jitc->function->call.size, JIT_SP, regno);
+	    _jitc->function->call.size += sizeof(jit_word_t);
+	}
+#endif
 	++_jitc->function->call.argi;
     }
     else {
 	regno = jit_get_reg(jit_class_gpr);
 	jit_movi(regno, u);
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size &
+		((1 << ((code - jit_code_pushargr_c) >> 2)) - 1);
+	switch (code) {
+	    case jit_code_pushargi_c:	case jit_code_pushargi_uc:
+		jit_stxi_c(_jitc->function->call.size, JIT_SP, regno);
+		break;
+	    case jit_code_pushargi_s:	case jit_code_pushargi_us:
+		jit_stxi_s(_jitc->function->call.size, JIT_SP, regno);
+		break;
+#  if __WORDSIZE == 32
+	    case jit_code_pushargi_i:
+		jit_stxi(_jitc->function->call.size, JIT_SP, regno);
+		break;
+#  else
+	    case jit_code_pushargi_i:	case jit_code_pushargi_ui:
+		jit_stxi_i(_jitc->function->call.size, JIT_SP, regno);
+		break;
+	    case jit_code_pushargi_l:
+		jit_stxi(_jitc->function->call.size, JIT_SP, regno);
+		break;
+#  endif
+	    default:
+		abort();
+		break;
+	}
+	_jitc->function->call.size += 1 << ((code - jit_code_pushargr_c) >> 2);
+#else
 	jit_stxi(_jitc->function->call.size, JIT_SP, regno);
-	jit_unget_reg(regno);
 	_jitc->function->call.size += sizeof(jit_word_t);
+#endif
+	jit_unget_reg(regno);
     }
     jit_dec_synth();
 }
@@ -677,11 +923,26 @@ _jit_pushargr_f(jit_state_t *_jit, jit_int32_t u)
     jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
 	jit_movr_f(JIT_FA0 - _jitc->function->call.argf, u);
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(sizeof(jit_float32_t) == sizeof(jit_word_t));
+	    jit_stxi_f(_jitc->function->call.size, JIT_SP,
+		       JIT_FA0 - _jitc->function->call.argf);
+	    _jitc->function->call.size += sizeof(jit_word_t);
+	}
+#endif
 	++_jitc->function->call.argf;
     }
     else {
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size & (sizeof(jit_float32_t) - 1);
+	jit_stxi_f(_jitc->function->call.size, JIT_SP, u);
+	_jitc->function->call.size += sizeof(jit_float32_t);
+#else
 	jit_stxi_f(_jitc->function->call.size, JIT_SP, u);
 	_jitc->function->call.size += sizeof(jit_word_t);
+#endif
     }
     jit_dec_synth();
 }
@@ -695,14 +956,29 @@ _jit_pushargi_f(jit_state_t *_jit, jit_float32_t u)
     jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
 	jit_movi_f(JIT_FA0 - _jitc->function->call.argf, u);
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(sizeof(jit_float32_t) == sizeof(jit_word_t));
+	    jit_stxi_f(_jitc->function->call.size, JIT_SP,
+		       JIT_FA0 - _jitc->function->call.argf);
+	    _jitc->function->call.size += sizeof(jit_word_t);
+	}
+#endif
 	++_jitc->function->call.argf;
     }
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_f(regno, u);
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size & (sizeof(jit_float32_t) - 1);
+	jit_stxi_f(_jitc->function->call.size, JIT_SP, regno);
+	_jitc->function->call.size += sizeof(jit_float32_t);
+#else
  	jit_stxi_f(_jitc->function->call.size, JIT_SP, regno);
-	jit_unget_reg(regno);
 	_jitc->function->call.size += sizeof(jit_word_t);
+#endif
+	jit_unget_reg(regno);
     }
     jit_dec_synth();
 }
@@ -715,11 +991,23 @@ _jit_pushargr_d(jit_state_t *_jit, jit_int32_t u)
     jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
 	jit_movr_d(JIT_FA0 - _jitc->function->call.argf, u);
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(sizeof(jit_float64_t) == sizeof(jit_word_t));
+	    jit_stxi_d(_jitc->function->call.size, JIT_SP,
+		       JIT_FA0 - _jitc->function->call.argf);
+	    _jitc->function->call.size += sizeof(jit_float64_t);
+	}
+#endif
 	++_jitc->function->call.argf;
     }
     else {
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size & (sizeof(jit_float64_t) - 1);
+#endif
 	jit_stxi_d(_jitc->function->call.size, JIT_SP, u);
-	_jitc->function->call.size += sizeof(jit_word_t);
+	_jitc->function->call.size += sizeof(jit_float64_t);
     }
     jit_dec_synth();
 }
@@ -733,14 +1021,26 @@ _jit_pushargi_d(jit_state_t *_jit, jit_float64_t u)
     jit_link_prepare();
     if (jit_arg_f_reg_p(_jitc->function->call.argf)) {
 	jit_movi_d(JIT_FA0 - _jitc->function->call.argf, u);
+#if __APPLE__
+	if (_jitc->function->call.call & jit_call_varargs) {
+	    assert(sizeof(jit_float64_t) == sizeof(jit_word_t));
+	    jit_stxi_d(_jitc->function->call.size, JIT_SP,
+		       JIT_FA0 - _jitc->function->call.argf);
+	    _jitc->function->call.size += sizeof(jit_float64_t);
+	}
+#endif
 	++_jitc->function->call.argf;
     }
     else {
 	regno = jit_get_reg(jit_class_fpr);
 	jit_movi_d(regno, u);
+#if PACKED_STACK
+	_jitc->function->call.size +=
+	    _jitc->function->call.size & (sizeof(jit_float64_t) - 1);
+#endif
  	jit_stxi_d(_jitc->function->call.size, JIT_SP, regno);
 	jit_unget_reg(regno);
-	_jitc->function->call.size += sizeof(jit_word_t);
+	_jitc->function->call.size += sizeof(jit_float64_t);
     }
     jit_dec_synth();
 }
@@ -770,6 +1070,10 @@ _jit_finishr(jit_state_t *_jit, jit_int32_t r0)
     jit_node_t		*node;
     assert(_jitc->function);
     jit_inc_synth_w(finishr, r0);
+#if PACKED_STACK
+    _jitc->function->call.size +=
+	_jitc->function->call.size & (sizeof(jit_word_t) - 1);
+#endif
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     node = jit_callr(r0);
@@ -787,6 +1091,10 @@ _jit_finishi(jit_state_t *_jit, jit_pointer_t i0)
     jit_node_t		*node;
     assert(_jitc->function);
     jit_inc_synth_w(finishi, (jit_word_t)i0);
+#if PACKED_STACK
+    _jitc->function->call.size +=
+	_jitc->function->call.size & (sizeof(jit_word_t) - 1);
+#endif
     if (_jitc->function->self.alen < _jitc->function->call.size)
 	_jitc->function->self.alen = _jitc->function->call.size;
     node = jit_calli(i0);
@@ -835,10 +1143,15 @@ void
 _jit_retval_i(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_inc_synth_w(retval_i, r0);
+#if __WORDSIZE == 32
+    jit_movr(r0, JIT_RET);
+#else
     jit_extr_i(r0, JIT_RET);
+#endif
     jit_dec_synth();
 }
 
+#if __WORDSIZE == 64
 void
 _jit_retval_ui(jit_state_t *_jit, jit_int32_t r0)
 {
@@ -851,10 +1164,10 @@ void
 _jit_retval_l(jit_state_t *_jit, jit_int32_t r0)
 {
     jit_inc_synth_w(retval_l, r0);
-    if (r0 != JIT_RET)
-	jit_movr(r0, JIT_RET);
+    jit_movr(r0, JIT_RET);
     jit_dec_synth();
 }
+#endif
 
 void
 _jit_retval_f(jit_state_t *_jit, jit_int32_t r0)
@@ -1496,11 +1809,23 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_live:			case jit_code_ellipsis:
 	    case jit_code_va_push:
 	    case jit_code_allocai:		case jit_code_allocar:
-	    case jit_code_arg:
+	    case jit_code_arg_c:		case jit_code_arg_s:
+	    case jit_code_arg_i:
+#  if __WORDSIZE == 64
+	    case jit_code_arg_l:
+#  endif
 	    case jit_code_arg_f:		case jit_code_arg_d:
 	    case jit_code_va_end:
 	    case jit_code_ret:
-	    case jit_code_retr:			case jit_code_reti:
+	    case jit_code_retr_c:		case jit_code_reti_c:
+	    case jit_code_retr_uc:		case jit_code_reti_uc:
+	    case jit_code_retr_s:		case jit_code_reti_s:
+	    case jit_code_retr_us:		case jit_code_reti_us:
+	    case jit_code_retr_i:		case jit_code_reti_i:
+#if __WORDSIZE == 64
+	    case jit_code_retr_ui:		case jit_code_reti_ui:
+	    case jit_code_retr_l:		case jit_code_reti_l:
+#endif
 	    case jit_code_retr_f:		case jit_code_reti_f:
 	    case jit_code_retr_d:		case jit_code_reti_d:
 	    case jit_code_getarg_c:		case jit_code_getarg_uc:
@@ -1508,16 +1833,34 @@ _emit_code(jit_state_t *_jit)
 	    case jit_code_getarg_i:		case jit_code_getarg_ui:
 	    case jit_code_getarg_l:
 	    case jit_code_getarg_f:		case jit_code_getarg_d:
-	    case jit_code_putargr:		case jit_code_putargi:
+	    case jit_code_putargr_c:		case jit_code_putargi_c:
+	    case jit_code_putargr_uc:		case jit_code_putargi_uc:
+	    case jit_code_putargr_s:		case jit_code_putargi_s:
+	    case jit_code_putargr_us:		case jit_code_putargi_us:
+	    case jit_code_putargr_i:		case jit_code_putargi_i:
+#if __WORDSIZE == 64
+	    case jit_code_putargr_ui:		case jit_code_putargi_ui:
+	    case jit_code_putargr_l:		case jit_code_putargi_l:
+#endif
 	    case jit_code_putargr_f:		case jit_code_putargi_f:
 	    case jit_code_putargr_d:		case jit_code_putargi_d:
-	    case jit_code_pushargr:		case jit_code_pushargi:
+	    case jit_code_pushargr_c:		case jit_code_pushargi_c:
+	    case jit_code_pushargr_uc:		case jit_code_pushargi_uc:
+	    case jit_code_pushargr_s:		case jit_code_pushargi_s:
+	    case jit_code_pushargr_us:		case jit_code_pushargi_us:
+	    case jit_code_pushargr_i:		case jit_code_pushargi_i:
+#if __WORDSIZE == 64
+	    case jit_code_pushargr_ui:		case jit_code_pushargi_ui:
+	    case jit_code_pushargr_l:		case jit_code_pushargi_l:
+#endif
 	    case jit_code_pushargr_f:		case jit_code_pushargi_f:
 	    case jit_code_pushargr_d:		case jit_code_pushargi_d:
 	    case jit_code_retval_c:		case jit_code_retval_uc:
 	    case jit_code_retval_s:		case jit_code_retval_us:
 	    case jit_code_retval_i:
+#if __WORDSIZE == 64
 	    case jit_code_retval_ui:		case jit_code_retval_l:
+#endif
 	    case jit_code_retval_f:		case jit_code_retval_d:
 	    case jit_code_prepare:
 	    case jit_code_finishr:		case jit_code_finishi:
