@@ -2,10 +2,10 @@
 #define JIT_INSTR_MAX 224
     0,	/* data */
     0,	/* live */
-    0,	/* align */
+    48,	/* align */
     0,	/* save */
     0,	/* load */
-    0,  /* skip */
+    16,	/* skip */
     0,	/* #name */
     0,	/* #note */
     0,	/* label */
@@ -14,7 +14,10 @@
     0,	/* va_push */
     0,	/* allocai */
     0,	/* allocar */
-    0,	/* arg */
+    0,	/* arg_c */
+    0,	/* arg_s */
+    0,	/* arg_i */
+    0,	/* arg_l */
     0,	/* getarg_c */
     0,	/* getarg_uc */
     0,	/* getarg_s */
@@ -22,8 +25,20 @@
     0,	/* getarg_i */
     0,	/* getarg_ui */
     0,	/* getarg_l */
-    0,	/* putargr */
-    0,	/* putargi */
+    0,	/* putargr_c */
+    0,	/* putargi_c */
+    0,	/* putargr_uc */
+    0,	/* putargi_uc */
+    0,	/* putargr_s */
+    0,	/* putargi_s */
+    0,	/* putargr_us */
+    0,	/* putargi_us */
+    0,	/* putargr_i */
+    0,	/* putargi_i */
+    0,	/* putargr_ui */
+    0,	/* putargi_ui */
+    0,	/* putargr_l */
+    0,	/* putargi_l */
     16,	/* va_start */
     32,	/* va_arg */
     32,	/* va_arg_d */
@@ -97,12 +112,17 @@
     16,	/* movi */
     16,	/* movnr */
     16,	/* movzr */
+    48,	/* casr */
+    64,	/* casi */
     16,	/* extr_c */
     16,	/* extr_uc */
     16,	/* extr_s */
     16,	/* extr_us */
     16,	/* extr_i */
     16,	/* extr_ui */
+    32,	/* bswapr_us */
+    32,	/* bswapr_ui */
+    16,	/* bswapr_ul */
     32,	/* htonr_us */
     32,	/* htonr_ui */
     16,	/* htonr_ul */
@@ -195,13 +215,37 @@
     32,	/* callr */
     48,	/* calli */
     0,	/* prepare */
-    0,	/* pushargr */
-    0,	/* pushargi */
+    0,	/* pushargr_c */
+    0,	/* pushargi_c */
+    0,	/* pushargr_uc */
+    0,	/* pushargi_uc */
+    0,	/* pushargr_s */
+    0,	/* pushargi_s */
+    0,	/* pushargr_us */
+    0,	/* pushargi_us */
+    0,	/* pushargr_i */
+    0,	/* pushargi_i */
+    0,	/* pushargr_ui */
+    0,	/* pushargi_ui */
+    0,	/* pushargr_l */
+    0,	/* pushargi_l */
     0,	/* finishr */
     0,	/* finishi */
     0,	/* ret */
-    0,	/* retr */
-    0,	/* reti */
+    0,	/* retr_c */
+    0,	/* reti_c */
+    0,	/* retr_uc */
+    0,	/* reti_uc */
+    0,	/* retr_s */
+    0,	/* reti_s */
+    0,	/* retr_us */
+    0,	/* reti_us */
+    0,	/* retr_i */
+    0,	/* reti_i */
+    0,	/* retr_ui */
+    0,	/* reti_ui */
+    0,	/* retr_l */
+    0,	/* reti_l */
     0,	/* retval_c */
     0,	/* retval_uc */
     0,	/* retval_s */
@@ -401,9 +445,4 @@
     0,	/* movi_d_ww */
     16,	/* movr_d_w */
     32,	/* movi_d_w */
-    32,	/* bswapr_us */
-    32,	/* bswapr_ui */
-    16,	/* bswapr_ul */
-    48,	/* casr */
-    64,	/* casi */
 #endif /* __WORDSIZE */
